import {localeOptions} from '../../utils';

export default function field({t}) {
    return [
        {
            type: 'FieldInput',
            field: 'field',
            value: '',
            title: t('form.field'),
            warning: t('warning.field'),
        }, {
            type: 'LanguageInput',
            field: 'title',
            value: '',
            title: t('form.title'),
        }, {
            type: 'LanguageInput',
            field: 'info',
            value: '',
            title: t('form.info'),
        }, {
            type: 'SpanInput',
            field: 'formCreateCol>span',
            title: t('form.formItemSpan'),
        }, {
            type: 'ConfigItem',
            col: {show: true},
            style: 'margin-bottom: 10px',
            name: 'ignoreConfig',
            props: {
                label: t('form.ignore'),
                warning: t('warning.ignore'),
            },
            children: [{
                type: 'switch',
                field: 'ignore',
                value: false,
                wrap: {show: false},
                col: {show: false},
            }]
        },
        {
            type: 'ConfigItem',
            col: {show: true},
            name: 'labelConfig',
            style: 'margin-bottom: 10px',
            props: {
                label: t('form.formItem'),
            },
            children: [{
                type: 'HideConfig',
                field: 'formCreateWrap>title',
                col: {show: false},
                wrap: {show: false},
                value: true,
            }, {
                type: 'select',
                field: 'formCreateWrap>class',
                title: t('form.labelPosition'),
                col: {show: false},
                slot: 'append',
                options: localeOptions(t, [
                    {value: 'fc-wrap-top', label: 'top'},
                    {value: 'fc-wrap-left', label: 'left'},
                    {value: 'fc-wrap-right', label: 'right'},
                ]),
            }, {
                type: 'SizeInput',
                field: 'formCreateWrap>labelWidth',
                col: {show: false},
                value: '',
                slot: 'append',
                title: t('form.labelWidth'),
            }, {
                type: 'SizeInput',
                field: 'formCreateWrap>style>marginBottom',
                col: {show: false},
                value: '',
                slot: 'append',
                title: t('form.formItemMarginBottom'),
            }
            ]
        }, {
            type: 'Struct',
            field: '_control',
            name: 'control',
            value: [],
            title: t('form.control'),
            warning: t('form.controlDocument', {doc: '<a target="_blank" href="https://pro.form-create.com/doc/control" style="color: inherit;text-decoration: underline;">' + t('form.document') + '</a>'}),
            props: {
                defaultValue: [],
                validate(val) {
                    if (!Array.isArray(val)) return false;
                    if (!val.length) return true;
                    return !val.some(({rule}) => {
                        return !Array.isArray(rule);
                    });
                }
            }
        },
    ];
}
